# always use pre-built clang/llvm libs instead of runtime-built libs
%bcond_with local_clang_static

#lua jit not available for some architectures
%ifarch ppc64 aarch64 ppc64le
%bcond_with lua
%else
%bcond_without lua
%endif

# use --with shared to only link against libLLVM.so
%if 0%{?fedora} >= 28 || 0%{?rhel} > 7
%bcond_without llvm_shared
%else
%bcond_with llvm_shared
%endif

%bcond_without python2
# Build python3 support for distributions that have it
%if 0%{?fedora} >= 28 || 0%{?rhel} > 7
%bcond_without python3
%else
%bcond_with python3
%endif

%if %{with python3}
%global __python %{__python3}
%global python_bcc python3-bcc
%global python_cmds python2;python3
%else
%global __python %{__python2}
%global python_bcc python2-bcc
%global python_cmds python2
%endif

%if 0%{?anolis} >= 8
%define llvm_version 10.0.1
%define clang_version 10.0.1
%else
%define llvm_version 7.0.1
%define clang_version 7.0.1
%endif

%global obsoletd_bcc_version 0.8.0

Name:           bcc
Version:        0.16.0
Release:        1%{?dist}
Summary:        BPF Compiler Collection (BCC)
License:        ASL 2.0
URL:            https://github.com/iovisor/bcc
Source0:        %{url}/archive/v%{version}/%{name}-%{version}.tar.gz
Patch1:         0001-tests-test_array.py-add-clock_nanosleep-to-attach-po.patch
Patch2:         0002-fix-some-test-cases-failed-3235.patch
Patch3:         0003-test-test_histogram.py-fix-test-failed-on-kernel-5.1.patch
Patch4:         0004-aarch64-turn-off-jump-table-optimization-during-jit-.patch
Patch5:         0005-Netqtop-3037-3048.patch
Patch6:         0006-fix-python3-compatible-issue-for-netqtop.patch
Patch7:         0007-fix-netqtop-python3-compatible-3140.patch
Patch8:         0008-tools-netqtop-fix-rx-queue-id.patch

Patch1000:      1000-bcc-anolis-alibaba-cloud.patch

ExclusiveArch: x86_64 ppc64 aarch64 ppc64le
BuildRequires: util-linux
BuildRequires: bison cmake >= 2.8.7 flex make
BuildRequires: gcc gcc-c++ elfutils-libelf-devel-static
%if %{with python3}
BuildRequires: python3-devel
BuildRequires: python2 python2-rpm-macros
%else
BuildRequires: python2-devel
%endif
%if %{with lua}
BuildRequires: luajit luajit-devel
%endif

%if %{without local_clang_static}
BuildRequires: llvm-devel = %{llvm_version}
BuildRequires: clang-devel = %{clang_version}
BuildRequires: clang-libs = %{clang_version}
%if %{without llvm_shared}
BuildRequires: llvm-static = %{llvm_version}
%endif # without llvm_shared
%endif # without local_clang_static
BuildRequires: pkgconfig ncurses-devel
BuildRequires: kernel-headers >= 3.10.0-957


%description
Python bindings for BPF Compiler Collection (BCC). Control a BPF program from
userspace.


%if %{with lua}
%global lua_include `pkg-config --variable=includedir luajit`
%global lua_libs `pkg-config --variable=libdir luajit`/lib`pkg-config --variable=libname luajit`.so
%global lua_config -DLUAJIT_INCLUDE_DIR=%{lua_include} -DLUAJIT_LIBRARIES=%{lua_libs}
%endif

%prep
%setup -q -n bcc
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1

%patch1000 -p1
%build
mkdir build
pushd build
%cmake .. \
      -DCMAKE_BUILD_TYPE=RelWithDebInfo\
      -DCMAKE_CXX_FLAGS='-std=c++11' \
      -DCMAKE_INSTALL_PREFIX=/usr \
      -DREVISION_LAST=%{version} -DREVISION=%{version} \
      %{?lua_config} \
      -DPYTHON_CMD="%{python_cmds}" \
      %{?with_llvm_shared:-DENABLE_LLVM_SHARED=1}
%make_build
popd

%install
pushd build
%make_install
# mangle shebangs
find %{buildroot}/usr/share/bcc/{tools,examples} -type f -exec \
    sed -i -e '1 s|^#!/usr/bin/python$|#!'%{__python}'|' \
           -e '1 s|^#!/usr/bin/env python$|#!'%{__python}'|' {} \;
popd

# Move man pages to the right location
mkdir -p %{buildroot}%{_mandir}
mv %{buildroot}%{_datadir}/%{name}/man/* %{buildroot}%{_mandir}/
# Avoid conflict with other manpages
# https://bugzilla.redhat.com/show_bug.cgi?id=1517408
for i in `find %{buildroot}%{_mandir} -name "*.gz"`; do
  tname=$(basename $i)
  rename $tname %{name}-$tname $i
done
# Fix the symlink too
for i in `find %{buildroot}%{_mandir} -lname \*.gz` ; do
    target=`readlink $i`;
    ln -sf bcc-$target $i;
done
#mkdir -p %{buildroot}%{_docdir}/%{name}
#mv %{buildroot}%{_datadir}/%{name}/examples %{buildroot}%{_docdir}/%{name}/

%package -n libbcc
Summary: Shared Library for BPF Compiler Collection (BCC)
Requires: elfutils-libelf
Requires: clang-libs = %{clang_version}
Obsoletes: bcc <= %{obsoletd_bcc_version}
Obsoletes: bcc-devel <= %{obsoletd_bcc_version}
Obsoletes: ebcc ebcc-devel

%description -n libbcc
Shared Library for BPF Compiler Collection (BCC)

%if %{with python2}
%package -n python2-bcc
Summary: Python2 bindings for BPF Compiler Collection (BCC)
Requires: libbcc = %{version}-%{release}
%{?python_provide:%python_provide python2-bcc}
Obsoletes: python-bcc <= %{obsoletd_bcc_version}
Obsoletes: python-ebcc

%description -n python2-bcc
Python bindings for BPF Compiler Collection (BCC)
%endif

%if %{with python3}
%package -n python3-bcc
Summary: Python3 bindings for BPF Compiler Collection (BCC)
Requires: libbcc = %{version}-%{release}
%{?python_provide:%python_provide python3-bcc}

%description -n python3-bcc
Python bindings for BPF Compiler Collection (BCC)
%endif

%if %{with lua}
%package -n bcc-lua
Summary: Standalone tool to run BCC tracers written in Lua
Requires: libbcc = %{version}-%{release}

%description -n bcc-lua
Standalone tool to run BCC tracers written in Lua
%endif

%package -n libbcc-examples
Summary: Examples for BPF Compiler Collection (BCC)
Requires: %{python_bcc} = %{version}-%{release}
%if %{with lua}
Requires: bcc-lua = %{version}-%{release}
%endif
Obsoletes: ebcc-doc

%description -n libbcc-examples
Examples for BPF Compiler Collection (BCC)

%package -n bcc-tools
Summary: Command line tools for BPF Compiler Collection (BCC)
Requires: %{python_bcc} = %{version}-%{release}
Obsoletes: ebcc-tools ebcc-osstat

%description -n bcc-tools
Command line tools for BPF Compiler Collection (BCC)


%files -n libbcc
/usr/lib64/*
/usr/include/bcc/*

%if %{with python2}
%files -n python2-bcc
%{python2_sitelib}/bcc*
%endif

%if %{with python3}
%files -n python3-bcc
%{python3_sitelib}/bcc*
%endif

%if %{with lua}
%files -n bcc-lua
/usr/bin/bcc-lua
%endif

%files -n libbcc-examples
/usr/share/bcc/examples/*
%if %{without lua}
%exclude /usr/share/bcc/examples/lua
%exclude /usr/share/bcc/examples/lua/*
%endif

%files -n bcc-tools
/usr/share/bcc/introspection/*
/usr/share/bcc/tools/*
%{_mandir}/man8/bcc-*

%post -n libbcc -p /sbin/ldconfig

%postun -n libbcc -p /sbin/ldconfig

%changelog
* Wed Sep 08 2021 Hang Zhao <1209176326@qq.com> - 0.16.0-1
- Update to bcc-0.16.0-8.al7.alnx

* Mon Jan 28 2019 Jerome Marchand <jmarchan@redhat.com> - 0.8.0-1
- Rebase on bcc-8.0.0
- Update libLLVM.so name for 7.0

* Fri Sep 21 2018 Jerome Marchand <jmarchan@redhat.com> - 0.6.1-2
- Set a minimal version for llvm-private(-devel)

* Thu Aug 16 2018 Jerome Marchand <jmarchan@redhat.com> - 0.6.1-1
- Rebase on v0.6.1
- Fix tcpsubnet
- Reinstate llcstat tool
- Remove inject tool
- Add NSS support to sslsniff
- Fixes miscellaneous error uncovered by covscan

* Wed Jul 04 2018 Jerome Marchand <jmarchan@redhat.com> - 0.6.0-3
- Fix tools on RHEL 7
- Remove llcstat and ZFS tools.

* Tue Jun 26 2018 Jerome Marchand <jmarchan@redhat.com> - 0.6.0-2
- Add llvm-private requirement
- Fix manpages symlinks

* Tue Jun 19 2018 Jerome Marchand <jmarchan@redhat.com> - 0.6.0-1
- Rebase on bcc-0.6.0

* Thu Jun 07 2018 Jerome Marchand <jmarchan@redhat.com> - 0.5.0-6
- Enables build on RHEL 7

* Thu May 24 2018 Jerome Marchand <jmarchan@redhat.com> - 0.5.0-5
- Enables build on ppc64(le) and s390x arches

* Thu Apr 05 2018 Rafael Santos <rdossant@redhat.com> - 0.5.0-4
- Resolves #1555627 - fix compilation error with latest llvm/clang

* Wed Feb 07 2018 Fedora Release Engineering <releng@fedoraproject.org> - 0.5.0-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Fri Feb 02 2018 Igor Gnatenko <ignatenkobrain@fedoraproject.org> - 0.5.0-2
- Switch to %%ldconfig_scriptlets

* Wed Jan 03 2018 Rafael Santos <rdossant@redhat.com> - 0.5.0-1
- Rebase to new released version

* Thu Nov 16 2017 Rafael Santos <rdossant@redhat.com> - 0.4.0-4
- Resolves #1517408 - avoid conflict with other manpages

* Thu Nov 02 2017 Rafael Santos <rdossant@redhat.com> - 0.4.0-3
- Use weak deps to not require lua subpkg on ppc64(le)

* Wed Nov 01 2017 Igor Gnatenko <ignatenkobrain@fedoraproject.org> - 0.4.0-2
- Rebuild for LLVM5

* Wed Nov 01 2017 Rafael Fonseca <rdossant@redhat.com> - 0.4.0-1
- Resolves #1460482 - rebase to new release
- Resolves #1505506 - add support for LLVM 5.0
- Resolves #1460482 - BPF module compilation issue
- Partially address #1479990 - location of man pages
- Enable ppc64(le) support without lua
- Soname versioning for libbpf by ignatenkobrain

* Wed Aug 02 2017 Fedora Release Engineering <releng@fedoraproject.org> - 0.3.0-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Wed Jul 26 2017 Fedora Release Engineering <releng@fedoraproject.org> - 0.3.0-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Thu Mar 30 2017 Igor Gnatenko <ignatenko@redhat.com> - 0.3.0-2
- Rebuild for LLVM4
- Trivial fixes in spec

* Fri Mar 10 2017 Rafael Fonseca <rdossant@redhat.com> - 0.3.0-1
- Rebase to new release.

* Fri Feb 10 2017 Fedora Release Engineering <releng@fedoraproject.org> - 0.2.0-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_Mass_Rebuild

* Tue Jan 10 2017 Rafael Fonseca <rdossant@redhat.com> - 0.2.0-2
- Fix typo

* Tue Nov 29 2016 Rafael Fonseca <rdossant@redhat.com> - 0.2.0-1
- Initial import
